unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Dialogs,
  Menus, StdCtrls, ComCtrls, ShellAPI, Grids, MPHexEditor, ExtCtrls,
  Clipbrd;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    MM_File: TMenuItem;
    MenuOpen: TMenuItem;
    N1:      TMenuItem;
    MenuExit: TMenuItem;
    OpenDlg: TOpenDialog;
    StatusBar: TStatusBar;
    Tabs:    TTabControl;
    StartupPanel: TPanel;
    MM_Help: TMenuItem;
    MenuAbout: TMenuItem;
    HexDump: TMPHexEditor;
    HexDumpMenu: TPopupMenu;
    HD_Worminate: TMenuItem;
    HexPanel: TPanel;
    StringsPanel: TPanel;
    StringsOptions: TPanel;
    CheckASCII: TCheckBox;
    CheckUnicode: TCheckBox;
    EditLength: TEdit;
    LabelSTR: TLabel;
    StringsList: TListBox;
    Populate: TButton;
    LabelError: TLabel;
    LabelTypeERR: TLabel;
    StringsMenu: TPopupMenu;
    S_Worminate: TMenuItem;
    S_FollowInHD: TMenuItem;
    LabelOptions: TLabel;
    HD_Copy: TMenuItem;
    MM_View: TMenuItem;
    MM_View_FileOFfset: TMenuItem;
    MM_View_FileOffset_Hex: TMenuItem;
    MM_View_FileOffset_Decimal: TMenuItem;
    HD_Copy_Raw: TMenuItem;
    HD_Copy_Hex: TMenuItem;
    MM_Edit: TMenuItem;
    MM_Edit_Copy: TMenuItem;
    MM_Edit_Copy_Raw: TMenuItem;
    MM_Edit_Copy_Hex: TMenuItem;
    MM_Edit_Find: TMenuItem;
    procedure MM_Edit_FindClick(Sender: TObject);
    procedure Openit(Filename: string);
    procedure MenuOpenClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure TabsChange(Sender: TObject);
    procedure MM_View_FileOffset_HexClick(Sender: TObject);
    procedure MM_View_FileOffset_DecimalClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
    procedure HexDumpMenuPopup(Sender: TObject);
    procedure HD_WorminateClick(Sender: TObject);
    procedure EditLengthChange(Sender: TObject);
    procedure PopulateClick(Sender: TObject);
    procedure CheckExit(Sender: TObject);
    procedure CheckKeyUp(Sender: TObject; var Key: word; Shift: TShiftState);
    procedure S_FollowInHDClick(Sender: TObject);
    procedure EditLengthExit(Sender: TObject);
    procedure S_WorminateClick(Sender: TObject);
    procedure StringsMenuPopup(Sender: TObject);
    procedure StartupPanelClick(Sender: TObject);
    procedure HD_Copy_RawClick(Sender: TObject);
    procedure HD_Copy_HexClick(Sender: TObject);
    procedure MM_EditClick(Sender: TObject);
  protected
    procedure WMDROPFILES(var Msg: TMessage); message WM_DROPFILES;
  private
    function IsPrintable(character: char): boolean;
  end;

var
  MainForm: TMainForm;
  StringsOffsets: TStringList;

implementation

uses About, Base64, Find;

{$R *.DFM}

procedure TMainForm.WMDROPFILES(var Msg: TMessage);
var
  Size:     integer;
  Filename: PChar;
begin
  inherited;
  Size     := DragQueryFile(Msg.WParam, 0, nil, 0) + 1;
  Filename := StrAlloc(Size);
  DragQueryFile(Msg.WParam, 0, Filename, Size);
  Openit(Filename);
  StrDispose(Filename);
  DragFinish(Msg.WParam);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  DragAcceptFiles(MainForm.Handle, True);
  StringsOffsets := TStringList.Create;
  if ParamCount > 0 then
  begin
    Openit(ParamStr(1));
  end;
end;

procedure TMainForm.Openit(Filename: string);
var
  filecheck:     file of byte;
  filechecksize: integer;
begin
  try
    Screen.Cursor := crHourglass;
    StatusBar.SimpleText := 'Reading...';
    Application.ProcessMessages;
    try
      AssignFile(filecheck, Filename);
      FileMode := fmOpenRead;
      Reset(filecheck);
      filechecksize := FileSize(filecheck);
      CloseFile(filecheck);
      if (filechecksize > 10485760) then
      begin
        raise EOutOfResources.Create('');
      end;
      HexDump.LoadFromFile(Filename);
      StatusBar.SimpleText := ExtractFileName(Filename);
      Tabs.Visible := True;
      StartupPanel.Free;
    except
      on E: EFOpenError do
      begin
        StatusBar.SimpleText := '';
        MessageDlg(Format('Can''t open file %s.', [StatusBar.SimpleText]),
          mtError, [mbOK], 0);
      end;
      on E: EOutOfResources do
      begin
        StatusBar.SimpleText := '';
        MessageDlg('Files larger than 10mb are not supported by this version.',
          mtError, [mbOK], 0);
      end;
    end;
  finally
    MainMenu.Items.Items[1].Enabled := True;
    MainMenu.Items.Items[2].Enabled := True;
    StringsList.Clear;
    Screen.Cursor := crDefault;
  end;
end;


procedure TMainForm.MenuOpenClick(Sender: TObject);
begin
  if OpenDlg.Execute then
  begin
    Openit(OpenDlg.FileName);
  end;
end;

procedure TMainForm.MenuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.TabsChange(Sender: TObject);
begin
  if Tabs.TabIndex = 1 then
  begin
    HexPanel.Visible     := False;
    StringsPanel.Visible := True;
    MM_Edit_Copy.Enabled := False;
  end
  else
  begin
    HexPanel.Visible     := True;
    StringsPanel.Visible := False;
    MM_Edit_Copy.Enabled := True;
  end;
end;

procedure TMainForm.MM_View_FileOffset_HexClick(Sender: TObject);
begin
  HexDump.OffsetFormat := '8!10:|';
  HexDump.Update;
  MM_View_FileOffset_Hex.Checked := True;
end;

procedure TMainForm.MM_View_FileOffset_DecimalClick(Sender: TObject);
begin
  HexDump.OffsetFormat := '8!0A:|';
  HexDump.Update;
  MM_View_FileOffset_Decimal.Checked := True;
end;

procedure TMainForm.MenuAboutClick(Sender: TObject);
begin
  AboutForm.Show;
end;

procedure TMainForm.HexDumpMenuPopup(Sender: TObject);
begin
  if (HexDump.SelectionAsText = '') then
  begin
    HD_Worminate.Enabled := False;
    HD_Copy.Enabled      := False;
  end
  else
  begin
    HD_Worminate.Enabled := True;
    HD_Copy.Enabled      := True;
  end;
end;

procedure TMainForm.HD_WorminateClick(Sender: TObject);
begin
  Base64Form.input := HexDump.SelectionAsText;
  Base64Form.Show;
end;

procedure TMainForm.EditLengthChange(Sender: TObject);
var
  I:    integer;
  B, L: boolean;
begin
  B := True;
  L := length(EditLength.Text) > 0;
  if L then
  begin
    for I := 1 to length(EditLength.Text) do
    begin
      if not (EditLength.Text[I] in ['0'..'9']) then
      begin
        LabelError.Visible := True;
        Populate.Enabled := False;
        B := False;
      end;
    end;
  end;
  if not L then
  begin
    LabelError.Visible := True;
    Populate.Enabled   := False;
  end;
  if L and B then
  begin
    if StrToInt(EditLength.Text) > HexDump.DataSize then
    begin
      LabelError.Visible := True;
      Populate.Enabled   := False;
    end
    else
    begin
      LabelError.Visible := False;
      Populate.Enabled   := True;
    end;
  end;
end;

procedure TMainForm.PopulateClick(Sender: TObject);
var
  I, printable: integer;
  str: string;
begin
  StringsList.Clear;
  StringsOffsets.Clear;

  if CheckASCII.Checked then
  begin
    I := 1;
    while (I < HexDump.DataSize) do
    begin
      printable := 0;
      str := '';
      while IsPrintable(chr(HexDump.Data[I])) and (I < HexDump.Datasize) do
      begin
        printable := printable + 1;
        str := str + chr(HexDump.Data[I]);
        I   := I + 1;
      end;
      if printable >= StrToInt(EditLength.Text) then
      begin
        StringsList.Items.Append(str);
        StringsOffsets.Add(IntToStr(I - length(str)));
      end;
      I := I + 1;
    end;
  end;

  if CheckUnicode.Checked then
  begin
    I := 1;
    while (I < HexDump.DataSize - 1) do
    begin
      printable := 0;
      str := '';
      while IsPrintable(chr(HexDump.Data[I])) and (I < HexDump.Datasize - 1) and
        (HexDump.Data[I + 1] = 0) do
      begin
        printable := printable + 1;
        str := str + chr(HexDump.Data[I]);
        I   := I + 2;
      end;
      if printable >= StrToInt(EditLength.Text) then
      begin
        StringsList.Items.Append(str);
        StringsOffsets.Add(IntToStr(I - length(str) * 2));
      end;
      I := I + 1;
    end;
  end;
end;

function TMainForm.IsPrintable(character: char): boolean;
begin
  Result := False;
  case character of
    'a'..'z':
    begin
      Result := True;
    end;
    'A'..'Z':
    begin
      Result := True;
    end;
    '0'..'9':
    begin
      Result := True;
    end;
    '/', '-', ':', '.', ',', '_', '$', '#', '%', '\',
    '(', ')', '[', ']', '<', '>', '{', '}', '?', '"',
    '''', ';', '`', '~', '!', '@', '^', '&', '*', '=',
    '+', ' ':
    begin
      Result := True;
    end;
  end;
end;

procedure TMainForm.CheckExit(Sender: TObject);
begin
  if not (CheckUnicode.Checked or CheckASCII.Checked) then
  begin
    LabelTypeErr.Visible := True;
    Populate.Enabled     := False;
  end
  else
  begin
    LabelTypeErr.Visible := False;
    Populate.Enabled     := True;
  end;
end;

procedure TMainForm.CheckKeyUp(Sender: TObject; var Key: word; Shift: TShiftState);
begin
  CheckExit(Sender);
end;

procedure TMainForm.S_FollowInHDClick(Sender: TObject);
begin
  HexDump.SelStart := StrToInt(StringsOffsets[StringsList.ItemIndex]);
  HexDump.SelEnd   := HexDump.SelStart +
    Length(StringsList.Items[StringsList.ItemIndex]) - 1;
  if (HexDump.Data[HexDump.SelStart + 1] = $00) then
  begin
    HexDump.SelEnd := HexDump.SelEnd + Length(StringsList.Items[StringsList.ItemIndex]);
  end;
  Tabs.TabIndex := 0;
  TabsChange(Sender);
  HexDump.SetFocus;
end;

procedure TMainForm.EditLengthExit(Sender: TObject);
begin
  if (Populate.Enabled) and (StrToInt(EditLength.Text) < 3) then
  begin
    ShowMessage('Minimum allowed length is 3');
    EditLength.Text := '3';
  end;
end;

procedure TMainForm.S_WorminateClick(Sender: TObject);
var
  I: integer;
begin
  Base64Form.input := '';
  if (HexDump.Data[StrToInt(StringsOffsets[StringsList.ItemIndex]) + 1] = $00) then
    for I := 1 to length(StringsList.Items[StringsList.ItemIndex]) do
      Base64Form.input := Base64Form.input +
        StringsList.Items[StringsList.ItemIndex][I] + #00
  else
    Base64Form.input := StringsList.Items[StringsList.ItemIndex];
  Base64Form.Show;
end;

procedure TMainForm.StringsMenuPopup(Sender: TObject);
begin
  if StringsList.ItemIndex = -1 then
  begin
    S_Worminate.Enabled  := False;
    S_FollowInHD.Enabled := False;
  end
  else
  begin
    S_Worminate.Enabled  := True;
    S_FollowInHD.Enabled := True;
  end;
end;

procedure TMainForm.StartupPanelClick(Sender: TObject);
begin
  MenuOpenClick(Sender);
end;

procedure TMainForm.HD_Copy_RawClick(Sender: TObject);
begin
  Clipboard.AsText := HexDump.SelectionAsText;
end;

procedure TMainForm.HD_Copy_HexClick(Sender: TObject);
begin
  Clipboard.AsText := HexDump.SelectionAsHex;
end;

procedure TMainForm.MM_EditClick(Sender: TObject);
begin
  if (HexPanel.Visible and (HexDump.SelectionAsText <> '')) then
  begin
    MM_Edit_Copy.Enabled := True;
  end
  else
  begin
    MM_Edit_Copy.Enabled := False;
  end;
end;

procedure TMainForm.MM_Edit_FindClick(Sender: TObject);
begin
  FindForm.Show;
end;

end.
